from .base import DNSProvider


class Arvan(DNSProvider):
    base_url = "https://napi.arvancloud.com/cdn/4.0/domains"

    def get_record(self, domain, record_type):
        # Implement Arvan's specific API calls
        pass

    def create_record(self, record):
        # Implement Arvan's specific API calls
        pass

    def update_record(self, record):
        # Implement Arvan's specific API calls
        pass

    def delete_record(self, record):
        # Implement Arvan's specific API calls
        pass



from .base import DNSProvider


class Cloudflare(DNSProvider):
    base_url = "https://api.cloudflare.com/client/v4/zones"

    def get_record(self, domain, record_type):
        # Implement Cloudflare's specific API calls
        pass

    def create_record(self, record):
        # Implement Cloudflare's specific API calls
        pass

    def update_record(self, record):
        # Implement Cloudflare's specific API calls
        pass

    def delete_record(self, record):
        # Implement Cloudflare's specific API calls
        pass



import abc


class DNSProvider(abc.ABC):
    def __init__(self, api_key):
        self.api_key = api_key

    @abc.abstractmethod
    def get_record(self, domain, record_type):
        pass

    @abc.abstractmethod
    def create_record(self, record):
        pass

    @abc.abstractmethod
    def update_record(self, record):
        pass

    @abc.abstractmethod
    def delete_record(self, record):
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['auto_dns', 'auto_dns.dns_providers']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0.1,<7.0.0', 'requests>=2.31.0,<3.0.0', 'typer>=0.9.0,<0.10.0']

entry_points = \
{'console_scripts': ['autodns = auto_dns:app']}

setup_kwargs = {
    'name': 'auto-dns',
    'version': '0.1.0',
    'description': 'A DNS Python Library.',
    'long_description': '===================\nAuto DNS Python CLI\n===================\n\nAuto DNS is a Python command line application which automatically updates the DNS records of your domain using various DNS providers.\n\nSupported DNS providers:\n\n- Arvan\n- Cloudflare\n\nThis library is intended to be used on a system with a dynamic public IP that you want to map to a static domain name.\n\nInstallation\n============\n\nThe package can be installed using pip:\n\n.. code-block:: shell\n\n   $ pip install auto-dns\n\nOr if you prefer, you can use Poetry:\n\n.. code-block:: shell\n\n   $ poetry add auto-dns\n\nConfiguration\n=============\n\nTo configure a DNS provider API key, use the following command:\n\n.. code-block:: shell\n\n   $ autodns set_api_key <provider> <api_key>\n\nUsage\n=====\n\nTo update a DNS record with your current public IP, use the following command:\n\n.. code-block:: shell\n\n   $ autodns update <domain> <record_type> <provider>\n\nContributing\n============\n\nContributions are welcome! Please feel free to submit a Pull Request.\n\nLicense\n=======\n\nAuto DNS is released under the MIT License.\n',
    'author': 'Ali Tavallaie',
    'author_email': 'a.tavallaie@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import os
import yaml
import requests
import typer
from auto_dns.dns_providers.arvan import Arvan
from auto_dns.dns_providers.cloudflare import Cloudflare
from auto_dns.exceptions import UnsupportedProviderError


class DNSRecord:
    def __init__(self, domain, record_type, ip):
        self.domain = domain
        self.record_type = record_type
        self.ip = ip


class DDNS:
    def __init__(self, provider):
        self.provider = provider

    def get_record(self, domain, record_type):
        return self.provider.get_record(domain, record_type)

    def create_record(self, record):
        self.provider.create_record(record)

    def update_record(self, record):
        self.provider.update_record(record)

    def delete_record(self, record):
        self.provider.delete_record(record)


DEFAULT_CONFIG_PATH = os.path.join(os.path.expanduser("~"), ".auto_ddns", "config.yaml")


def get_public_ip():
    """Get the public IP address of the device."""
    try:
        response = requests.get("https://api.ipify.org")
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException as error:
        print(f"Error occurred when getting public IP: {error}")
        raise


def write_config(provider_name: str, api_key: str, config_file: str):
    config = {}
    if os.path.exists(config_file):
        with open(config_file, "r") as file:
            config = yaml.safe_load(file) or {}
    config[provider_name] = {"api_key": api_key}
    with open(config_file, "w") as file:
        yaml.dump(config, file)


def get_provider(provider_name: str, config_file: str):
    if not os.path.exists(config_file):
        raise FileNotFoundError(f"Config file not found: {config_file}")
    with open(config_file) as file:
        config = yaml.safe_load(file)
    if provider_name == "arvan":
        return Arvan(config["arvan"]["api_key"])
    elif provider_name == "cloudflare":
        return Cloudflare(config["cloudflare"]["api_key"])
    else:
        raise UnsupportedProviderError(f"Unsupported provider: {provider_name}")


app = typer.Typer()


@app.command()
def set_api_key(provider: str, api_key: str, config_file: str = DEFAULT_CONFIG_PATH):
    """Set the API key for a provider."""
    if provider not in ["arvan", "cloudflare"]:
        raise UnsupportedProviderError(f"Unsupported provider: {provider}")
    write_config(provider, api_key, config_file)


@app.command()
def get(
    domain: str, record_type: str, provider: str, config_file: str = DEFAULT_CONFIG_PATH
):
    provider_instance = get_provider(provider, config_file)
    ddns = DDNS(provider_instance)
    print(ddns.get_record(domain, record_type))


@app.command()
def create(
    domain: str,
    record_type: str,
    provider: str,
    ip: str = None,
    config_file: str = DEFAULT_CONFIG_PATH,
):
    provider_instance = get_provider(provider, config_file)
    ddns = DDNS(provider_instance)
    ip = ip or get_public_ip()
    record = DNSRecord(domain, record_type, ip)
    ddns.create_record(record)


@app.command()
def update(
    domain: str,
    record_type: str,
    provider: str,
    ip: str = None,
    config_file: str = DEFAULT_CONFIG_PATH,
):
    provider_instance = get_provider(provider, config_file)
    ddns = DDNS(provider_instance)
    ip = ip or get_public_ip()
    record = DNSRecord(domain, record_type, ip)
    ddns.update_record(record)


@app.command()
def delete(
    domain: str, record_type: str, provider: str, config_file: str = DEFAULT_CONFIG_PATH
):
    provider_instance = get_provider(provider, config_file)
    ddns = DDNS(provider_instance)
    record = DNSRecord(
        domain, record_type, "192.168.1.1"
    )  # placeholder IP for delete operation
    ddns.delete_record(record)


if __name__ == "__main__":
    app()
